#!/usr/bin/env python

# Copyright (C)2002 Thomas Mangin
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; version 2.1 of the License
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

'''
This package offer an easy way to manipulate netstring.
For more information about netstring look at :
http://cr.yp.to/proto/netstrings.txt

It used property and as such needs python version 2.2 or later

Example of usage:

	n = netstring('optional text')
	print n
	print n.len
	print n.string
	print n.netstring

	n.netstring='15:A new netstring,')
	n.string='Another way to initialize')
	
'''

revision = '$Revision: 0.60 $'


def version_check():
	import sys
	if sys.version[0] < 2 or (sys.version[0] == 2 and sys.version[1] < 2):
		raise "This module require python 2.2 or later"
version_check()


import string
import socket, select
import types
import os


def __split(netstring):
	"return a tuple with the position of the : and the size of the string"
	pos = string.find(netstring, ':')
	if pos < 1: raise ValueError, 'Missing semicolon'

	try: size = int(netstring[:pos])
	except: raise ValueError, 'Bad numeric component'

	return (pos,size)

def length(netstring):
	"check the netstring integrity and return its size"
	try: (pos,size)=__split(netstring)
	except: return -1

	if pos+size+2 != len(netstring): return -1
	if netstring[-1] != ',': return -1

	return size

class netstring(object):
	"A class to manipulate netstring"

	#__slots__ = ('string', 'len')

	def __init__(self,value=''):
		self.__set_string(value)

	def __get_string(self):
		return self.__string

	def __set_string(self,value):
		self.__len=len(value)
		self.__string=value

	def __get_len(self):
		return self.__len

	def __set_len(self,value):
		self.__len=value

	def __get_netstring(self):
		return ('%d:%s,'%(self.__len,self.__string))

	def __set_netstring(self,nstring):
		size = length(nstring)
		
		if size < 0: raise ValueError, "Bad string"
		self.string = nstring[-size-1:-1]

		return self

	string = property(__get_string,__set_string,None,"The netstring string")
	len = property(__get_len,__set_len,None,"The length of the netstring")
	netstring = property(__get_netstring,__set_netstring,None,"")

	def __repr__(self):
		return self.__get_netstring()

	def __len__(self):
		return self.len

	def __add__(self,other):
		if isinstance(other,self.__class__):
			return self.__class__(self.string + other.string)
		else:
			raise ValueError, 'Bad type for concatenation'

	def tuple(self):
		return (self.__len,self.__string)


if __name__ == "__main__":
	import sys

	def leave():
		print "self test failed, leaving"
		sys.exit(100)

	print "testing netstring class"
	s1 = netstring()
	s2 = netstring('test1')

	if str(s1) != '0:,': leave()
	if str(s2) != '5:test1,': leave()
	if s2.string != 'test1' : leave()

	s1.string='test2'
	if str(s1) != '5:test2,': leave()

	s1.netstring="6:thomas,"
	if str(s1) != '6:thomas,': leave()

	try:
		s1.netstring="10:thomas,"
		leave()
	except: pass
	try:
		s1.netstring="10:tho"
		leave()
	except: pass
	try:
		s1.netstring="4:thomas,"
		leave()
	except: pass
	try:
		s1.netstring=":thomas,"
		leave()
	except: pass

	s1.string='1+1'
	s2.string='=2'

	if ((s1+s2).netstring != '5:1+1=2,'): leave()

	s1.netstring='3:2+2,'
	s2.netstring='2:=4,'

	if ((s1+s2).netstring != '5:2+2=4,'): leave()

	print "self test successed, leaving"
	sys.exit(0)

